@@ -5,9 +5,9 @@
     Copyright (C) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
 
     Rewritten for QT4 by e_k <e_k at users.sourceforge.net>, Copyright (C)2008
-    Copyright (C) 2012-2013 Jacob Dawid <jacob.dawid@googlemail.com>
+    Copyright (C) 2012-2019 Jacob Dawid <jacob.dawid@cybercatalyst.com>
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
@@ -41,6 +41,7 @@
 #include <QPixmap>
 #include <QScrollBar>
 #include <QStyle>
+#include <QToolTip>
 #include <QtCore>
 #include <QtGui>
 
@@ -89,7 +90,7 @@ void TerminalView::setScreenWindow(ScreenWindow* window)
   // disconnect existing screen window if any
   if ( _screenWindow )
     {
-      disconnect( _screenWindow , 0 , this , 0 );
+      disconnect( _screenWindow , nullptr , this , nullptr );
     }
 
   _screenWindow = window;
@@ -212,6 +213,12 @@ void TerminalView::setVTFont(const QFont& f)
       // Disabling kerning saves some computation when rendering text.
       // font.setKerning(false);
 
+      QFont::StyleStrategy strategy = font.styleStrategy();
+#if defined (HAVE_QFONT_FORCE_INTEGER_METRICS)
+      strategy |= QFont::ForceIntegerMetrics;
+#endif
+      font.setStyleStrategy(QFont::StyleStrategy(strategy));
+
       QWidget::setFont(font);
       fontChange(font);
     }
@@ -230,9 +237,9 @@ void TerminalView::setFont(const QFont &)
 
 TerminalView::TerminalView(QWidget *parent)
   :QWidget(parent)
-  ,_screenWindow(0)
+  ,_screenWindow(nullptr)
   ,_allowBell(true)
-  ,_gridLayout(0)
+  ,_gridLayout(nullptr)
   ,_fontHeight(1)
   ,_fontWidth(1)
   ,_fontAscent(1)
@@ -242,7 +249,7 @@ TerminalView::TerminalView(QWidget *parent)
   ,_usedColumns(1)
   ,_contentHeight(1)
   ,_contentWidth(1)
-  ,_image(0)
+  ,_image(nullptr)
   ,_randomSeed(0)
   ,_resizing(false)
   ,_terminalSizeHint(false)
@@ -262,9 +269,9 @@ TerminalView::TerminalView(QWidget *parent)
   ,_tripleClickMode(SelectWholeLine)
   ,_isFixedSize(false)
   ,_possibleTripleClick(false)
-  ,_resizeWidget(0)
-  ,_resizeTimer(0)
-  ,_outputSuspendedLabel(0)
+  ,_resizeWidget(nullptr)
+  ,_resizeTimer(nullptr)
+  ,_outputSuspendedLabel(nullptr)
   ,_lineSpacing(0)
   ,_colorsInverted(false)
   ,_blendColor(qRgba(0,0,0,0xff))
@@ -293,9 +300,15 @@ TerminalView::TerminalView(QWidget *parent)
   // setup timers for blinking cursor and text
   _blinkTimer   = new QTimer(this);
   connect(_blinkTimer, SIGNAL(timeout()), this, SLOT(blinkEvent()));
+
   _blinkCursorTimer   = new QTimer(this);
   connect(_blinkCursorTimer, SIGNAL(timeout()), this, SLOT(blinkCursorEvent()));
 
+  _process_filter_timer = new QTimer (this);
+  connect (_process_filter_timer, SIGNAL (timeout ()),
+           this, SLOT (processFilters ()));
+  _process_filter_timer->start (300);
+
   //  QCursor::setAutoHideCursor( this, true );
 
   setUsesMouse(true);
@@ -322,6 +335,8 @@ TerminalView::TerminalView(QWidget *parent)
 
   connect (this, SIGNAL (set_global_shortcuts_signal (bool)),
            parent->parent (), SLOT (set_global_shortcuts (bool)));
+  connect (this, SIGNAL (set_global_shortcuts_signal (bool)),
+           parent, SLOT (set_global_shortcuts (bool)));
 
 }
 
@@ -704,11 +719,12 @@ void TerminalView::scrollImage(int lines , const QRect& screenWindowRegion)
   QRect region = screenWindowRegion;
   region.setBottom( qMin(region.bottom(),this->_lines-2) );
 
-  if (    lines == 0
-          || _image == 0
-          || !region.isValid()
-          || (region.top() + abs(lines)) >= region.bottom()
-          || this->_lines <= region.height() ) return;
+  if (lines == 0
+      || _image == nullptr
+      || !region.isValid()
+      || (region.top() + abs(lines)) >= region.bottom()
+      || this->_lines <= region.height() )
+    return;
 
   QRect scrollRect;
 
@@ -817,7 +833,7 @@ void TerminalView::updateImage()
   _screenWindow->resetScrollCount();
 
   Character* const newimg = _screenWindow->getImage();
-  int lines = _screenWindow->windowLines() + 1;
+  int lines = _screenWindow->windowLines();
   int columns = _screenWindow->windowColumns();
 
   setScroll( _screenWindow->currentLine() , _screenWindow->lineCount() );
@@ -1185,27 +1201,33 @@ void TerminalView::paintFilters(QPainter& painter)
                        endColumn*_fontWidth - 1, (line+1)*_fontHeight - 1 );
 
           // Underline link hotspots
-          if ( spot->type() == Filter::HotSpot::Link )
+          if ( spot->type() == Filter::Link ||
+               spot->type() == Filter::ErrorLink)
             {
               QFontMetrics metrics(font());
 
-              // find the baseline (which is the invisible line that the characters in the font sit on,
-              // with some having tails dangling below)
-              int baseline = r.bottom() - metrics.descent();
+              // find the baseline (which is the invisible line that the
+              // characters in the font sit on
+              int baseline = r.bottom() + 1;
               // find the position of the underline below that
               int underlinePos = baseline + metrics.underlinePos();
 
-              if ( r.contains( mapFromGlobal(QCursor::pos()) ) )
-                painter.drawLine( r.left() , underlinePos ,
-                                  r.right() , underlinePos );
+              if (r.contains (mapFromGlobal(QCursor::pos())))
+                {
+                  if (spot->type () == Filter::ErrorLink)
+                    painter.setPen (QColor (255,0,0));
+                  painter.drawLine (r.left(), underlinePos,
+                                    r.right() + 2, underlinePos);
+                }
             }
           // Marker hotspots simply have a transparent rectanglular shape
           // drawn on top of them
-          else if ( spot->type() == Filter::HotSpot::Marker )
+          else if ( spot->type() == Filter::Marker )
             {
               //TODO - Do not use a hardcoded colour for this
               painter.fillRect(r,QBrush(QColor(255,0,0,120)));
             }
+
         }
     }
 }
@@ -1548,8 +1570,59 @@ void TerminalView::mousePressEvent(QMouseEvent* ev)
   getCharacterPosition(ev->pos(),charLine,charColumn);
   QPoint pos = QPoint(charColumn,charLine);
 
+  // reposition cursor if mouseclick happens in the currently editable line
+  QVector<LineProperty> lineprop = _screenWindow->getLineProperties ();
+  int lineStart = _screenWindow->cursorPosition ().y ();
+  if (lineStart > 0)
+    {
+      while (lineprop[lineStart-1] & LINE_WRAPPED)
+        lineStart--;
+    }
+
+  if (charLine >= lineStart)
+    {
+      int posInLineCursor = _screenWindow->cursorPosition ().x ()
+                            + qMax (0,(_screenWindow->cursorPosition ().y ()-lineStart))
+                            * _screenWindow->windowColumns ();
+      int posInLineClick = charColumn
+                           + qMax (0, (charLine - lineStart))
+                           * _screenWindow->windowColumns ();
+
+      if (posInLineCursor > posInLineClick)
+        {
+          QKeyEvent *event = new QKeyEvent (QEvent::KeyPress,Qt::Key_Left,
+                                            Qt::NoModifier,QString (""));
+
+          for (int i = 0; i < posInLineCursor - posInLineClick; i++)
+            emit keyPressedSignal (event);
+
+          delete event;
+        }
+      else if (posInLineClick > posInLineCursor)
+        {
+          QKeyEvent *event = new QKeyEvent (QEvent::KeyPress,Qt::Key_Right,
+                                            Qt::NoModifier,QString (""));
+
+          for (int i = 0; i < posInLineClick - posInLineCursor; i++)
+            emit keyPressedSignal (event);
+
+          delete event;
+        }
+    }
+
   if ( ev->button() == Qt::LeftButton)
     {
+
+      Filter::HotSpot* spot = _filterChain->hotSpotAt(charLine,charColumn);
+      if ( spot &&
+          (spot->type() == Filter::Link || spot->type() == Filter::ErrorLink))
+        {
+          QList<QAction*> actions = spot->actions ();
+          if (actions.length ())
+            actions.at (0)->activate (QAction::Trigger);
+          return;
+        }
+
       _lineSelectionMode = false;
       _wordSelectionMode = false;
 
@@ -1634,8 +1707,13 @@ void TerminalView::mouseMoveEvent(QMouseEvent* ev)
   // handle filters
   // change link hot-spot appearance on mouse-over
   Filter::HotSpot* spot = _filterChain->hotSpotAt(charLine,charColumn);
-  if ( spot && spot->type() == Filter::HotSpot::Link)
+  if ( spot &&
+      (spot->type() == Filter::Link || spot->type() == Filter::ErrorLink))
     {
+      // change mouse cursor when mouse is over links
+      if (! _mouseOverHotspotArea.isValid())
+        setCursor (Qt::PointingHandCursor);
+
       QRect previousHotspotArea = _mouseOverHotspotArea;
       _mouseOverHotspotArea.setCoords( qMin(spot->startColumn() , spot->endColumn()) * _fontWidth,
                                        spot->startLine() * _fontHeight,
@@ -1654,6 +1732,7 @@ void TerminalView::mouseMoveEvent(QMouseEvent* ev)
     }
   else if ( _mouseOverHotspotArea.isValid() )
     {
+      setUsesMouse (true);
       update( _mouseOverHotspotArea );
       // set hotspot area to an invalid rectangle
       _mouseOverHotspotArea = QRect();
@@ -2266,7 +2345,7 @@ void TerminalView::setSelection(const QString& t)
   QApplication::clipboard()->setText(t, QClipboard::Selection);
 }
 
-void TerminalView::copyClipboard()
+void TerminalView::copyClipboard(bool extra_interrupt)
 {
   if ( !_screenWindow || !hasFocus())
     return;
@@ -2274,7 +2353,10 @@ void TerminalView::copyClipboard()
   QString text = _screenWindow->selectedText(_preserveLineBreaks);
 
   if (text.isEmpty ())
-    emit interrupt_signal ();
+    {
+      if (! extra_interrupt)
+        emit interrupt_signal ();
+    }
   else
     QApplication::clipboard()->setText(text);
 }
@@ -2287,11 +2369,26 @@ void TerminalView::pasteClipboard()
     }
 }
 
+void TerminalView::selectAll()
+{
+  if ( !_screenWindow || !hasFocus())
+    return;
+
+  _screenWindow->setSelectionStart(0,-_screenWindow->currentLine(), false);
+  //_screenWindow->setSelectionEnd(_screenWindow->windowColumns(),
+  //                               _screenWindow->windowLines());
+
+  _screenWindow->setSelectionEnd(_screenWindow->columnCount(),
+                                 _screenWindow->windowLines());
+}
+
+
 void TerminalView::pasteSelection()
 {
   emitSelection(true,false);
 }
 
+
 /* ------------------------------------------------------------------------- */
 /*                                                                           */
 /*                                Keyboard                                   */
@@ -2361,7 +2458,56 @@ void TerminalView::keyPressEvent( QKeyEvent* event )
     }
 
   if ( emitKeyPressSignal && !_readonly )
-    emit keyPressedSignal(event);
+    {
+      // Clear selection if the cursor is moved with arrow keys
+      bool emitKey = true;
+      if ((event->modifiers() == Qt::NoModifier)
+          && (event->key() == Qt::Key_Right || event->key() == Qt::Key_Left))
+        _screenWindow->clearSelection();
+
+      // Delete selected text if printable key is pressed
+      if ( !event->text ().isEmpty () )
+        {
+          int lineStart, colStart, lineEnd, colEnd;
+          _screenWindow->getSelectionStart (colStart, lineStart);
+          _screenWindow->getSelectionEnd (colEnd, lineEnd);
+          int cursorLoc = loc (_screenWindow->cursorPosition ().x (),
+                               _screenWindow->cursorPosition ().y ());
+
+          //Check if there is a selection in the current line
+          if ((loc (colStart,lineStart) != loc (colEnd, lineEnd))
+               && (cursorLoc >=loc (colStart,lineStart))
+               && (cursorLoc-1 <= loc (colEnd,lineEnd)))
+            {
+              QKeyEvent *ev = new QKeyEvent (QEvent::KeyPress,Qt::Key_Backspace,
+                                             Qt::NoModifier,QString (""));
+
+              if (cursorLoc < loc (colEnd, lineEnd))
+                {
+                  QKeyEvent *ev_right = new QKeyEvent (QEvent::KeyPress,Qt::Key_Right,
+                                                       Qt::NoModifier,QString (""));
+                  for (int i = 0; i < loc (colEnd,lineEnd) - cursorLoc + 1; i++)
+                    emit keyPressedSignal (ev_right);
+                  delete ev_right;
+                }
+
+              for (int i = 0; i < loc (colEnd, lineEnd) - loc (colStart, lineStart) + 1; i++)
+                emit keyPressedSignal (ev);
+
+              delete ev;
+
+              // Backspace deleted the selected text and has done its duty, no need to call it again
+              if (event->key() == Qt::Key_Backspace)
+                {
+                  _screenWindow->clearSelection ();
+                  emitKey = false;
+                }
+            }
+        }
+
+      if (emitKey)
+        emit keyPressedSignal (event);
+    }
 
   if (_readonly) {
       event->ignore();
@@ -2369,6 +2515,7 @@ void TerminalView::keyPressEvent( QKeyEvent* event )
   else {
       event->accept();
     }
+
 }
 
 void TerminalView::inputMethodEvent( QInputMethodEvent* event )
@@ -2418,45 +2565,6 @@ QVariant TerminalView::inputMethodQuery( Qt::InputMethodQuery query ) const
   return QVariant();
 }
 
-bool TerminalView::event( QEvent *e )
-{
-  if ( e->type() == QEvent::ShortcutOverride )
-    {
-      QKeyEvent* keyEvent = static_cast<QKeyEvent *>( e );
-
-      // a check to see if keyEvent->text() is empty is used
-      // to avoid intercepting the press of the modifier key on its own.
-      //
-      // this is important as it allows a press and release of the Alt key
-      // on its own to focus the menu bar, making it possible to
-      // work with the menu without using the mouse
-      if ( (keyEvent->modifiers() == Qt::AltModifier) &&
-           !keyEvent->text().isEmpty() )
-        {
-          keyEvent->accept();
-          return true;
-        }
-
-      // Override any of the following shortcuts because
-      // they are needed by the terminal
-      int keyCode = keyEvent->key() | keyEvent->modifiers();
-      switch ( keyCode )
-        {
-        // list is taken from the QLineEdit::event() code
-        case Qt::Key_Tab:
-        case Qt::Key_Delete:
-        case Qt::Key_Home:
-        case Qt::Key_End:
-        case Qt::Key_Backspace:
-        case Qt::Key_Left:
-        case Qt::Key_Right:
-          keyEvent->accept();
-          return true;
-        }
-    }
-  return QWidget::event( e );
-}
-
 void TerminalView::setBellMode(int mode)
 {
   _bellMode=mode;
@@ -2606,6 +2714,17 @@ void TerminalView::dropEvent(QDropEvent* event)
   //  KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());
 
   QString dropText;
+
+  if (event->mimeData ()->hasUrls ())
+  {
+    foreach (QUrl url, event->mimeData ()->urls ())
+    {
+      if(dropText.length () > 0)
+        dropText += '\n';
+      dropText  += url.toLocalFile ();
+    }
+  }
+
   /*  if (!urls.isEmpty())
   {
     for ( int i = 0 ; i < urls.count() ; i++ )
@@ -2710,3 +2829,14 @@ QString TerminalView::selectedText ()
   QString text = _screenWindow->selectedText (_preserveLineBreaks);
   return text;
 }
+
+void
+TerminalView::visibility_changed (bool visible)
+{
+  // Disable the timer for processing the filter chain, since this might time
+  // consuming
+  if (visible)
+    _process_filter_timer->start (300);
+  else
+    _process_filter_timer->stop ();
+}
