@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.2.  */
+/* A Bison parser, made by GNU Bison 3.2.4.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,25 +30,36 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_OCTAVE_TEX_COREFCN_OCT_TEX_PARSER_H_INCLUDED
-# define YY_OCTAVE_TEX_COREFCN_OCT_TEX_PARSER_H_INCLUDED
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
+#ifndef YY_OCTAVE_TEX_LIBINTERP_COREFCN_OCT_TEX_PARSER_H_INCLUDED
+# define YY_OCTAVE_TEX_LIBINTERP_COREFCN_OCT_TEX_PARSER_H_INCLUDED
 /* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
+#ifndef OCTAVE_TEX_DEBUG
+# if defined YYDEBUG
 #if YYDEBUG
+#   define OCTAVE_TEX_DEBUG 1
+#  else
+#   define OCTAVE_TEX_DEBUG 0
+#  endif
+# else /* ! defined YYDEBUG */
+#  define OCTAVE_TEX_DEBUG 0
+# endif /* ! defined YYDEBUG */
+#endif  /* ! defined OCTAVE_TEX_DEBUG */
+#if OCTAVE_TEX_DEBUG
 extern int octave_tex_debug;
 #endif
 /* "%code requires" blocks.  */
-#line 53 "../../libinterp/corefcn/oct-tex-parser.yy" /* yacc.c:1909  */
+#line 59 "/home/jwe/src/octave-stable/libinterp/corefcn/oct-tex-parser.yy" /* yacc.c:1906  */
 #include <string>
 
-#line 47 "corefcn/oct-tex-parser.h" /* yacc.c:1909  */
+#line 58 "libinterp/corefcn/oct-tex-parser.h" /* yacc.c:1906  */
 
 /* Token type.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
+#ifndef OCTAVE_TEX_TOKENTYPE
+# define OCTAVE_TEX_TOKENTYPE
+  enum octave_tex_tokentype
   {
     BF = 258,
     IT = 259,
@@ -89,32 +100,34 @@ extern int octave_tex_debug;
 #define STR 274
 
 /* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE YYSTYPE;
-union YYSTYPE
+#if ! defined OCTAVE_TEX_STYPE && ! defined OCTAVE_TEX_STYPE_IS_DECLARED
+
+union OCTAVE_TEX_STYPE
 {
-#line 55 "../../libinterp/corefcn/oct-tex-parser.yy" /* yacc.c:1909  */
+#line 62 "/home/jwe/src/octave-stable/libinterp/corefcn/oct-tex-parser.yy" /* yacc.c:1906  */
 
-  /* Leaf symbols produced by the scanner */
-  char                       ch;
-  double                     num;
-  int                        sym;
+  // Leaf symbols produced by the scanner.
+  char ch;
+  double num;
+  int sym;
 
-  /* Used for string buffering */
-  std::string*               str;
+  // Used for string buffering.
+  std::string *str;
 
-  /* Objects produced by the parser */
-  text_element*              e_base;
-  text_element_list*         e_list;
+  // Objects produced by the parser.
+  octave::text_element *e_base;
+  octave::text_element_list *e_list;
 
-#line 111 "corefcn/oct-tex-parser.h" /* yacc.c:1909  */
+#line 122 "libinterp/corefcn/oct-tex-parser.h" /* yacc.c:1906  */
 };
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
+
+typedef union OCTAVE_TEX_STYPE OCTAVE_TEX_STYPE;
+# define OCTAVE_TEX_STYPE_IS_TRIVIAL 1
+# define OCTAVE_TEX_STYPE_IS_DECLARED 1
 #endif
 
 
 
-int octave_tex_parse (text_parser_tex& parser);
+int octave_tex_parse (octave::text_parser_tex& parser);
 
-#endif /* !YY_OCTAVE_TEX_COREFCN_OCT_TEX_PARSER_H_INCLUDED  */
+#endif /* !YY_OCTAVE_TEX_LIBINTERP_COREFCN_OCT_TEX_PARSER_H_INCLUDED  */
